# -*- coding:utf-8 -*-
# 创建时间：2018-12-14 17:03:43
# 创建人：  Dekiven

import os
from DKVTools.Funcs import *
from ftplib import FTP
import threading
import time

class FtpClient(FTP) :
    '''FtpClient
    '''

    def __init__(self, *args, **dArgs) :
        FTP.__init__(self, *args, **dArgs)
        self.host = None
        self.port = None
        self.userName = None
        self.password = None


    def connect(self, host='', port=0, userName='anonymous', password='anonymous@', timeout=None) :
        '''Connect to the given host and port. The default port number is 21, as specified by the FTP protocol specification. It is rarely needed to specify a different port number. This function should be called only once for each instance; it should not be called at all if a host was given when the instance was created. All other methods can only be used after a connection has been made. The optional timeout parameter specifies a timeout in seconds for the connection attempt. If no timeout is passed, the global default timeout setting will be used. source_address is a 2-tuple (host, port) for the socket to bind to as its source address before connecting.
Changed in version 3.3: source_address parameter was added.
        '''
        self.host = host
        self.port = port
        self.userName = userName
        self.password = password

        FTP.connect(self, host, port) #连接
        FTP.login(self, userName, password) #登录，如果匿名登录则用空串代替即可

    def setDebugLevel(self, level):
        # 级别2显示详细信息， 级别0关闭调试模式
        FTP.set_debuglevel(self, level) 

    def tryMkd(self, dirPath, relative2Root=False):
        curDir = self.pwd()[1:]
        if relative2Root :
            self.cwd('../'*len(curDir.split('/')))

        dirs = dirPath.split('/')
        for d in dirs :
            try:
                self.cwd(d)
            except Exception as e:
                try:
                    self.mkd(d)
                    self.cwd(d)
                except Exception as e:
                    print('makedir failed, please check your permission!')
                    # 还原 ftp 工作目录
                    if relative2Root :
                        self.cwd(curDir)
                    return
        if len(dirs) > 0 :
            self.cwd('../'*len(dirs))

        # 还原 ftp 工作目录
        if relative2Root :
            self.cwd(curDir)

    def ls(self, _dir='.') :
        '''返回指定路径的内容，默认为当前工作路径
返回内容格式:[[filename, permission, fileSize, modifyTime],... ]
fileSize 仅对文件有效，文件夹数据不正确
        '''
        import re

        p = re.compile(r'(\S+)\s+(\d+)\s+(.+)\s+(\d+)\s+(\S{3}\s+\d{2}\s+\d{2,}:?\d{2}?)\s+(\S.*)')

        lines = []
        self.dir(_dir, lines.append)

        content = []

        for l in lines :
            m = p.match(l)
            if m :
                content.append([m.group(6), m.group(1), m.group(4), m.group(5)])
            else :
                print('ls can not macth line: "%s"'%l)

        return content

    def walk(self, _dir='.') :
        ret = []
        fds = []
        fs = []
        for l in self.ls(_dir) :
            if l[1].startswith('d') :
                # 如果是文件夹
                d = l[0]
                fds.append(d)
                ret += self.walk(pathJoin(_dir, d))
            else :
                fs.append(l[0])
        ret.insert(0, [_dir, fds, fs])

        return ret

    #文件上传
    def upload(self, filePath, fname, blocksize=8192 , callback=None, mdk=True, ftp=None):
        ftp = ftp or self
        fname = formatPath(fname)
        if fname.startswith('/') :
            fname = fname[1:]

        if os.path.isfile(filePath) :
            size = os.path.getsize(filePath)
            count = [0,]
            def __c(content) :
                count[0] += len(content)
                if callback :
                    callback(min(1, count[0]/float(size)))
            fd = open(filePath, 'rb')
            try:
                d = os.path.split(fname)[0]
                if d != '' and mdk:
                    self.tryMkd(d)
                #以二进制的形式上传
                ftp.storbinary("STOR %s" % fname, fd, blocksize=blocksize, callback=__c)
                if size == 0 :
                    callback(1)
            except Exception as e:
                if callback :
                    callback(-1, e)
            finally:
                fd.close()
        # print("upload finished")

    #文件下载
    def download(self, fname, saveDir, callback=None, ftp=None) :
        ftp = ftp or self
        pdir = os.path.dirname(saveDir)
        if not os.path.isdir(pdir) :
            os.makedirs(pdir)
        fd = open(saveDir, 'wb')
        size = ftp.size(fname)
        count = [0,]
        def __c(content) :
            count[0] += len(content)
            fd.write(content)
            if callback :
                    callback(min(1, count[0]/float(size)))
        try:
            if size > 0 :
                #以二进制形式下载，注意第二个参数是fd.write，上传时是fd
                ftp.retrbinary("RETR %s" % fname, callback=__c)
        except Exception as e:
            if callback :
                callback(-1, e)
        finally:
            fd.close()  
        if size == 0 :
            callback(1)   

    def uploadDir(self, dirPath, ftpPath, callback=None, threadNum=4, isDaemon=False) :
        if not os.path.isdir(dirPath) :
            print('can not upload dir, dirPath :"%s" do not exist!!'%dirPath)
            return
        filePaths = []
        fNames = []
        newDirs = []

        for _dir, fds, fs in os.walk(dirPath) :
            relaPath = getRelativePath(_dir, dirPath)[0]
            if len(relaPath) > 0 :
                relaPath = pathJoin(ftpPath, relaPath)
            else :
                relaPath = ftpPath
            if relaPath not in newDirs :
                # TODO:优化减少不必要的 dir
                newDirs.append(relaPath)
            for f in fs :
                if f != '.DS_Store' :
                    filePaths.append(pathJoin(_dir, f))
                    fNames.append(pathJoin(relaPath, f))

        self.uploadFiles(filePaths, fNames, callback=callback, threadNum=threadNum)


    def uploadFiles(self, filePaths, fNames, callback=None, threadNum=4, newDirs=None, isDaemon=False) :
        if newDirs is None :
            newDirs = []
            for f in fNames :
                d = os.path.split(f)[0]
                if d != '' and d not in newDirs :
                    # TODO:优化减少不必要的 dir
                    newDirs.append(d)

        for d in newDirs :
            self.tryMkd(d)

        self.__startThread(filePaths, fNames, True, callback=callback, threadNum=threadNum, isDaemon=isDaemon)

    def downloadDir(self, ftpPath, savePath, callback=None, threadNum=4, isDaemon=False) :
        ftpPaths = []
        savePaths = []
        savePath = savePath.rstrip('/')
        for _dir, fds, fs in self.walk(ftpPath) :
            for f in fs :
                p = pathJoin(_dir, f)
                ftpPaths.append(p)
                savePaths.append(p.replace(ftpPath, savePath))

        self.downloadFiles(ftpPaths, savePaths, callback=callback, threadNum=threadNum)

    def downloadFiles(self, ftpPaths, savePaths, callback=None, threadNum=4, isDaemon=False) :
        self.__startThread(ftpPaths, savePaths, False, callback=callback, threadNum=threadNum, isDaemon=isDaemon)

    def __getMove2Next(self, arg1, arg2) :

        lock = threading.Lock()
        data = [-1, arg1, arg2]

        def move2Next() :
            lock.acquire()
            data[0] += 1
            a1, a2 = None, None
            idx = data[0]
            if idx < len(data[1]) and len(data[1]) == len(data[2]) :
                a1, a2 = data[1][idx], data[2][idx]
            lock.release()
            return a1, a2

        return move2Next

    def __startThread(self, arg1, arg2, isUp, callback=None, threadNum=4, isDaemon=False) :
        move2Next = self.__getMove2Next(arg1, arg2)

        def downloadThread() :
            data = [None] * 3
            ftp = FTP()
            ftp.connect(self.host, self.port)
            ftp.login(self.userName, self.password)

            def call(progress, exception=None) :
                if callback :
                    callback(data[1], progress, exception)
                data[2] = progress

                if cmp(abs(progress), 1) == 0 :
                    path, name = move2Next()
                    data[0] = path
                    data[1] = name
                    data[2] = None
    
            path, name = move2Next()
            data[0] = path
            data[1] = name
            data[2] = None
            
            while True :
                path, name = data[0], data[1]
                if data[0] is not None and data[2] is None :
                    data[2] = 0
                    if isUp :
                        self.upload(path, name, mdk=False, callback=call, ftp=ftp)
                    else :
                        self.download(path, name, callback=call, ftp=ftp)
                elif data[0] is None :
                    break
                time.sleep(1)
            ftp.quit()

        for i in range(threadNum) :
            t = threading.Thread(target=downloadThread)
            t.setDaemon(isDaemon)
            t.start()
        # downloadThread()

def __main() :
    # print('Module named:'+str(FtpClient))
    help(FTP)
if __name__ == '__main__' :
    __main()



# -*- coding: UTF-8 -*-
# 创建时间：2018年3月9日21:29:09
# 创建人：  Dekiven_PC


import os
import sys
import platform
import zipfile
import shutil
# import subprocess
# import tempfile
'''
    常用工具函数
    python2 中的unicode -> python3 的str
    python2 中的str-> python3 的byteArray
'''

def pathJoin(_dir, fn, abs=False) :
    return formatPath(os.path.join(_dir, fn), abs)

def isPython3():
    '''判断python版本是否是3.x'''
    return sys.version_info >= (3,0)

def isFunc(func):
    '''判断对象是否是函数'''
    return hasattr(func,'__call__')
    # return callable(func)

def getPlatform():
    '''获取python运行平台'''
    return platform.system()

def isWindows():
    '''判断是否是win32平台'''
    return getPlatform().lower() == 'windows'

def isLinux():
    '''判断是否是linux平台'''
    return getPlatform().lower() == 'linux'

def isMac():
    '''判断是否是Mac OS平台'''
    return getPlatform().lower() == 'darwin'

def getUserPath():
    return formatPath(os.path.expanduser('~'))

def getUserName():
    return os.path.split(getUserPath())[-1]

def getDeviceName() :
    import socket
    return socket.getfqdn(socket.gethostname())

def getLocalHost() :
    '''获取设备ip
    '''
    import socket
    ip = '127.0.0.1'
    try:
        s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        s.connect(('8.8.8.8', 80))
        ip = s.getsockname()[0]
    finally:
        s.close()
    return ip

def getDesktopPath():
    '''获取桌面的路径，只支持win32、mac'''
    # if isWindows():
    #     if isPython3():
    #         import winreg           # 内置模块，获取windows注册表
    #     else:
    #         import _winreg as winreg
    #     key = winreg.OpenKey(winreg.HKEY_CURRENT_USER, r'Software\Microsoft\Windows\CurrentVersion\Explorer\Shell Folders')
    #     return winreg.QueryValueEx(key, "Desktop")[0]
    if isWindows() or isMac():
        return '%s/Desktop'%(getUserPath())
    else:
        # TODO: Linux
        return ''

def addTempPathForOs(path):
    '''添加path到系统的PATH环境变量中，暂时只支持win32'''
    if isWindows():
        os.environ['PATH']+= ';' +path

def addTempPathForPy(path):
    '''添加path到python运行环境的PATH环境变量中'''
    sys.path.append(path)

def getMainCwd():
    '''获取入口脚本的文件夹路径'''
    return os.path.split(sys.argv[0])[0]

def bytes2utf8Str(s, encodingOri='GBK') :
    if (isPython3() and not isinstance(s, str)) or (not isPython3() and not isinstance(s, unicode)) :
        try :
            s = s.decode(encodingOri)
        except Exception as e :
            s = s.decode('utf-8')
    return s.encode('utf-8').decode('utf-8')


def zipFolder(folder, zipPath = None,  exts= '.*', skipFiles = (), inZipPath='', suffixs=None):
    '''压缩文件夹
    folder:待压缩的文件夹
    zipPath:压缩文件存储路径，默认为文件绝对夹路径加上后缀'.zip'
    exts:参与压缩的文件格式后缀，默认为'.*'，表示所有文件都参与压缩，多个后缀用英文逗号（','）隔开，如：'.zip,.txt'
    skipFiles:跳过压缩的文件列表（list或tuple）,在文件夹中的绝对路径
    inZipPath:将所有文件统一放到inZipPath文件夹下
    '''
    exts = suffixs or exts
    if not isinstance(folder, str) :
        print('error:\tfolder [%s] is not a string!'%(str(folder)))
        return
    if not os.path.isabs(folder) :
        folder = os.path.join(pyDir, folder)
    folderLen = len(folder)
    fileInfos = []
    exts = (exts == '.*') and exts or str(exts).lower().split(',')

    # list解析操作
    skip = [l.replace('\\', '/').strip() for l in skipFiles]
    if os.path.exists(folder) and os.path.isdir(folder) :
        for _dir, folders, files in os.walk(folder) :
            for f in files :
                ext = os.path.splitext(f)
                ext = str(ext[-1]).lower()
                p = os.path.join(_dir, f)
                # 获取zip文件夹内的路径，去掉前面的/
                absP = p[folderLen+1:].replace('\\', '/').strip()
                if len(inZipPath) > 0 :
                    absP = pathJoin(inZipPath, absP)
                if (exts == '.*' or ext in exts) and (not absP in skip):
                    fileInfos.append((p, absP))
                    skip.append(absP)
        if len(fileInfos) > 0 :
            if zipPath is None :
                zipPath = folder + '.zip'
            dirName = os.path.dirname(zipPath)
            if not os.path.exists(dirName) :
                os.makedirs(dirName)
            zipF = zipfile.ZipFile(zipPath, 'w', zipfile.ZIP_DEFLATED)
            # print(password)
            # zipF.setpassword(password)
            for f, af in fileInfos :
                zipF.write(f, af)
            zipF.close()
            print('zip [%s] finished! Total files: %d'%(zipPath, len(fileInfos)))
            return fileInfos
        else :
            print('no files need to zip!')

    else :
        print('error:\tfolder [%s] do not exists!'%(folder))

def tryMakeParentDir(path) :
    '''尝试将传入的地址的父目录创建出来'''
    d = os.path.dirname(path)
    if not os.path.isdir(d) :
        os.makedirs(d)

def tryMakeDir(path) :
    '''尝试将传入的目录创建出来'''
    if not os.path.isdir(path) :
        if not os.path.isfile(path) :
            os.makedirs(path)
        else :
            raise Exception('path "%s" has a file or folder'%path)

def copyTree(srcDir, targetDir, removeBefore=False, exts='.*', skipDirs=(), suffix=None) :
    '''Copy the srcDir to targetDir.
    参数: srcDir, targetDir, removeBeforee=False, exts='.*', skipDirs=()
    skipDirs 使用相对于srcDir的路径
    '''
    # 版本兼容
    exts = suffix or exts
    try :
        import win32api
        import win32con
    except Exception as e :
        win32api = None

    if not os.path.exists(srcDir):
        print('dir ["%s"] do not exists!'%(srcDir))
        return False
    srcDir = formatPath(srcDir)
    targetDir = formatPath(targetDir)
    if removeBefore and os.path.isdir(targetDir):
        # shutil.rmtree(targetDir)
        removeTree(targetDir)

    # 如果是文件，拷贝
    if os.path.isfile(srcDir) :
        # print('copyTree 1 ')
        suf = os.path.splitext(srcDir)[1]
        if not targetDir[-len(suf):] == suf :
            # 目标位置是路径
            targetDir = pathJoin(targetDir, os.path.split(srcDir)[-1])
            # print('copyTree 2 '+targetDir)
        # print('copyTree 3 ')
        dirname = os.path.dirname(targetDir)
        if not os.path.isdir(dirname) :
            os.makedirs(dirname)
        shutil.copyfile(srcDir, targetDir)
        return True

    # print('copy ["%s"] to ["%s"] ...'%(srcDir, targetDir))
    if exts != '.*' and isinstance(exts, str) :
            exts = [f.strip().lower() for f in exts.split(',')]
    elif isinstance(exts, list) or isinstance(exts, tuple) :
        exts = [s.lower() for s in exts]

    for curDir, folders, files in os.walk(srcDir) :
        # 跳过skipDirs定义的文件夹
        # print('relative dir:'+getRelativePath(curDir, srcDir)[0])
        relativeDir = getRelativePath(curDir, srcDir)[0].strip()
        for sd in skipDirs :
            if formatPath(relativeDir).find(sd) == 0:
                relativeDir = None
                break
        if relativeDir == None :
            continue

        for f in files :
            if exts == '.*' or str(os.path.splitext(f)[-1]).strip().lower() in exts:
                p = pathJoin(curDir, f)
                tp = pathJoin(targetDir, p.replace(srcDir, '')[1:])
                td = os.path.split(tp)[0]
                if not os.path.exists(td) :
                    # print('make '+td)
                    os.makedirs(td)
                if isWindows() and os.path.exists(tp) and win32api is not None :
                    # win32上只能通过下面的方式修改文件显隐性，隐藏文件覆盖会引发权限问题
                    win32api.SetFileAttributes(tp, win32con.FILE_ATTRIBUTE_NORMAL)
                shutil.copyfile(p, tp)
    # print('copy ["%s"] to ["%s"] \tsucceeded!'%(srcDir, targetDir))
    return True

def removeTree(path) :
    if os.path.isdir(path):
        # shutil.rmtree(path)
        if isWindows() :
            tryCmd('rd /s /q "%s"'%(path.replace('/', '\\')))
        else :
            tryCmd('rm -rf "%s"'%(path))
    elif os.path.isfile(path) :
        removeFile(path)

def removeFile(path, safe=True) :
    '''删除文件
    path：       文件路径
    safe=True:   安全删除，使用的文件不删除
    返回值: 1：成功          2：没有该文件          Exeption: 安全删除失败
    '''
    if os.path.isfile(path) :
        if safe :
            try :
                os.unlink(path)
            except Exception as e :
                return e
        else :
            os.remove(path)
        return 1
    return 0

def renameFile(ori, tar) :
    if os.path.isfile(ori) and ori != tar :
        d = os.path.dirname(tar)
        if not os.path.isdir(d) :
            os.makedirs(d)
        os.raname(ori, tar)
        return True
    return False


def openFile(path, mode, encoding='utf-8') :
    '''以指定格式打开文件，返回文件'''
    import codecs
    return codecs.open(path, mode, encoding=encoding)

def startFile(path) :
    '''图形系统打开文件或文件夹，目前仅测支持win32和mac
    return ： 如果成功打开文件返回True， 否则返回False
    '''
    if os.path.exists(path) :
        if isWindows() :
            os.startfile(path)
            return True
        elif isMac():
            # path = path.replace(' ', '" "')
            os.system('open "%s"'%(path,))
            return True
    return False

def selectInFolder( path ) :
    '''打开path父目录，如果是win32选中path指向的文件（夹）
    return ： 如果成功打开文件返回True， 否则返回False
    '''
    if os.path.exists(path) :
        p = path
        if isWindows() :
            os.system('explorer.exe /select, "%s"'%path.replace('/', '\\'))
        else :
            startFile(os.path.dirname(path))
        return True

    return False

def getCounter(start = 1) :
    '''Get a counter, start is the value returned when first call the counter ,start is 0 if did not set
    '''
    c = [start-1]

    def count() :
        c[0] += 1
        return c[0]

    return count

def formatPath(path, abs=False) :
    path = str(path)
    if abs or os.path.exists(path) :
        path = os.path.abspath(path)
    return path.replace('\\', '/')

def getRelativePath(oriPath, relative2) :
    '''return path, success
    path:       the path that oriPath relative to relative2 ,if oriPath do not satrt with relative2,return oriPath
    success:    if oriPath satrt with relative2
    '''
    oriPath = formatPath(oriPath)
    relative2 = formatPath(relative2)
    if oriPath.find(relative2) == 0 :
        return oriPath.replace(relative2, '').strip('/'), True
        # TODO:测试一下sub的方法，正常就用sub
        # return oriPath.sub(len(relative2)).strip('/'), True
    else :
        return oriPath, False

def modifyFile(path, oldStrs, newStrs) :
    '''modifyFile(path, oldStrs, newStrs)   通过replace修改文件内容，不会有多余的换行符生成
    oldStrs 文件中需要替换的字符串tuple或者list
    newStrs 文件中替换为的字符串tuple或者list，与oldStrs一一对应
    '''
    if os.path.isfile(path) :
        f = open(path, 'rb')
        content = bytes2utf8Str(f.read())
        content = content.replace('\r\n', '\n')
        f.close()
        if newStrs and oldStrs and len(newStrs) == len(oldStrs) :
            for i in range(len(newStrs)) :
                content = content.replace(oldStrs[i], newStrs[i])
        f = None
        if isPython3() :
            f = open(path, 'w', encoding='utf-8')
        else :
            f = open(path, 'w')
        f.write(content)
        f.close()

def tryCmd(cmd) :
    import subprocess
    import traceback
    import tempfile
    import locale

    cmd = cmd.encode(locale.getdefaultlocale()[1])
    if isPython3() :
        cmd = cmd.decode(locale.getdefaultlocale()[1])
    out_temp = None
    lines = []

    try:
        out_temp = tempfile.SpooledTemporaryFile()
        fileno = out_temp.fileno()
        obj = subprocess.Popen(cmd, stdout=fileno, stderr=fileno, shell=True)
        obj.wait()
        out_temp.seek(0)
        lines = out_temp.readlines()
        # print (lines)
    except Exception as e:
        print (traceback.format_exc())
        # raise e
        # pass
    finally:
        if out_temp:
            out_temp.close()
        return lines

def getPythonObjByStr(pythonStr) :
    '''将传入的字符串转化为python对象（函数、值等），
    返回值是一个作用域，可使用 getPythonObjByStr('xxxxx')['yy']访问'''
    scope = {}
    exec_code = compile(funcStr, '<string>', 'exec')
    exec(exec_code, scope)
    return scope

def getFuncByStr(funcStr, funcName) :
    '''根据传入的字符串和方法名获取定义的函数'''
    scope = {}
    exec_code = compile(funcStr, '<string>', 'exec')
    exec(exec_code, scope)
    return scope[funcName]

def jsonLoads(jsonStr, **params) :
    '''py2 py3 json.loads'''
    import json
    return json.loads(jsonStr, **params)

def jsonDumps(data, **params) :
    '''py2 py3 json.dumps
    可用参数及默认值：
        skipkeys=False,
        ensure_ascii=True,
        check_circular=True,
        allow_nan=True,
        cls=None,
        indent=None,
        separators=None,
        encoding="utf-8",
        default=None,
        sort_keys=False
    '''
    import json
    if isPython3() and 'encoding' in list(params.keys()) :
        del params['encoding']
    return json.dumps(data, **params)


def CamelCase2UnderLine(s):
    '''驼峰法命名转下划线命名'''
    newStr = ''
    for i in range(len(s)):
        c = s[i]
        v = ord(c)
        if v > 64 and v < 91 :
            if i > 0 and i < len(s) - 1 :
                preV = ord(s[i-1])
                if preV > 96 and preV < 123 :
                    newStr = newStr + '_'
                else :
                    nextV = ord(s[i+1])
                    if nextV > 96 and nextV < 123 and preV != 95:
                        newStr = newStr + '_'
            newStr = newStr + chr(v+32)
        else :
            newStr = newStr + c
    # print(newStr)
    return newStr

def UnderLine2CamelCase(s):
    '''下划线命名转驼峰法命名'''
    newStr = ''
    arr = s.split('_')
    while arr[0] == '' and len(arr) > 0 :
        arr.pop(0)
    if len(arr) > 1 :
        for i in range(1, len(arr)) :
            a = arr[i]
            if len(a) > 0 :
                arr[i] = a[0].upper() + a[1:]
    return ''.join(arr)

def isStr(s) :
    if isPython3() :
        return isinstance(s, str)
    else :
        return isinstance(s, str) or isinstance(s, unicode)

def equals(a, b) :
    if isPython3() :
        import operator
        return operator.eq(a, b)
    else :
        return cmp(a, b) == 0

def tryGetDictValue(dic, key, default=None, delete=False) :
    '''从dict中获取key对应的value,
default: 若不存在key则返回传入的默认值,默认 None
delete:  是否在获取到值后删除对应键值对,默认 False
    '''
    if isinstance(dic, dict) :
        if delete :
            return dic.pop(key, default)
        else :
            ret = dic.get(key)
            if ret != None :
                return ret
            else :
                return default
    else :
        return default

def getFileMd5(filename):
    import hashlib
    if not os.path.isfile(filename):
        return
    myhash = hashlib.md5()# create a md5 object
    with open(filename,'rb') as f :
        while True:
            b = f.read(8096)# get file content.
            if not b :
                break
            myhash.update(b)#encrypt the file
    return myhash.hexdigest()

def hasChineseChar(string) :
    import re
    if re.search('[\u4e00-\u9fa5]', string) :
        return True
    return False

def hasSpaceChar(string) :
    import re
    if re.search(r'\s', string) :
        return True
    return False

def pyData2LuaLines( data, key=None, deepth=0, indent='    ', lines=None, clearNil=True, strMarks='\'' ) :
    '''将python基本类型数据转换为lua格式'''
    if lines is None :
        lines = []
    _indent = indent * deepth
    line = _indent

    def append( l ) :
        '''去除多余的空行'''
        if len(l.strip()) > 0 :
            lines.append(l)


    if key is not None :
        if isinstance(key, int) or isinstance(key, float) :
            if key == int(key) :
                key = int(key)
            line = '%s[%s] = '%(_indent, key)
        else :
            # print(pattern.search(key), key)
            if pattern.search(key) :
                line = '%s%s = '%(_indent, str(key).strip())
            else :
                line = '%s[%s%s%s] = '%(_indent, strMarks, str(key).strip(), strMarks)

    if isinstance(data, dict) :
        append(line)
        append('%s{'%(_indent))
        for k,v in list(data.items()) :
            pyData2LuaLines(v, k, deepth+1, indent, lines, clearNil, strMarks)
        append('%s},'%(_indent))

    elif isinstance(data, list) or isinstance(data, tuple) :
        append(line)
        append('%s{'%(_indent))
        for v in data :
            pyData2LuaLines(v, None, deepth+1, indent, lines, clearNil, strMarks)
        append('%s},'%(_indent))

    elif type(data) == bool :
        line = '%s%s,'%(line, str(data).lower())
        append(line)

    elif isinstance(data, str) :
        line = '%s%s%s%s,'%(line, strMarks, data, strMarks)
        append(line)

    elif data is None :
        if not clearNil :
            line = '%s\'nil\','%(line,)
            append(line)

    else :
        line = '%s%s,'%(line, str(data))
        append(line)

    return lines

def delAllEmptySubFolders( path ) :
    '''删除指定路径下所有空文件夹'''
    if os.path.isdir(path) :
        for d, fds, fls in os.walk(path) :
            if len(fds) == 0 and len(fls) == 0 :
                delEmptyFoler(d)

def delEmptyFoler( path ) :
    '''如果传入文件夹为空则删除'''
    if os.path.isdir(path) :
        if len(os.listdir(path)) == 0 :
            removeTree(path)
            delEmptyFoler(os.path.dirname(path))

# ============================================线程相关===============================================================
# 见https://ituser.cn/python3-threading%E4%B9%8B%E7%BB%88%E6%AD%A2%E7%BA%BF%E7%A8%8B%E6%96%B9%E6%B3%95/
def __async_raise( tid, exctype ) :
    import threading
    import time
    import inspect
    import ctypes

    tid = ctypes.c_long(tid)
    if not inspect.isclass(exctype):
        exctype = type(exctype)
    res = ctypes.pythonapi.PyThreadState_SetAsyncExc(tid, ctypes.py_object(exctype))
    if res == 0:
        raise ValueError("invalid thread id")
    elif res != 1:
        # """if it returns a number greater than one, you're in trouble,
        # and you should call it again with exc=NULL to revert the effect"""
        ctypes.pythonapi.PyThreadState_SetAsyncExc(tid, None)
        raise SystemError("PyThreadState_SetAsyncExc failed")

def stopThread( thread ) :
    '''停止一个线程'''
    __async_raise(thread.ident, SystemExit)

# ===========================================================================================================

def Singleton(cls, *args, **kwds):
    '''单例修饰符函数，参考见：
    python 单例：http://ghostfromheaven.iteye.com/blog/1562618
    python @修饰符：http://blog.csdn.net/lainegates/article/details/8166764
    '''
    instances = {}
    def _singleton():
        if cls not in instances:
            instances[cls] = cls(*args, **kwds)
        return instances[cls]
    return _singleton

# ===========================================================================================================


# ==================================================测试========================================================
def __main() :
    # startFile(os.path.dirname(__file__))
    # startFile(getDesktopPath())
    # print(getLocalHost())
    # for s in ('as_dfg__h', '__as__b_df') :
    #     print(UnderLine2CamelCase(s), s)
    selectInFolder(r"F:/cocos2dx/cocostudio/Docs/Cocos/CocosProjects/G003_en_PLAZA/cocosstudio/Scene.udf")


if __name__ == '__main__':
    __main()


# -*- coding:utf-8 -*-

from setuptools import setup
from setuptools import find_packages
from DKVTools import __version__

setup(
    name='DKVTools',
    version=__version__,
    description=(
        'Some useful functions.'
    ),
    long_description='Some useful functions.',
    author='dekiven',
    author_email='dekiven@163.com',
    # maintainer='<维护人员的名字>',
    # maintainer_email='<维护人员的邮件地址',
    license='BSD License',
    packages=find_packages(),
    # 将包内资源包含到打的包中，貌似仅支持wheel
    include_package_data = True,
    install_requires=[
        # 'Pillow',
        # 'other>=1.1'
    ],
    data_files=[
    	# data_files的元素都是元组，元组的第一个元素是文件要放入的文件夹名称，第二个元素是文件列表。这里需要注意的是，如果不想把文件放入文件夹，可以将元组的第一个元素指定为空字符串，此时要打包的文件要被放入根目录，这里根目录是指python解释器所在的目录。比如我需要将文件资源放入python解释器所在目录下的/Lib/site-packages/myfolder路径，myfolder是自定义的文件夹，元组的第一个元素就可以写‘Lib/site-packages/myfolder’，打包时会自动在指定位置新建一个名为myfolder的目录，将文件资源放入其中。
        # 详见：https://www.cnblogs.com/lyrichu/p/6818008.html
    ],
    platforms=['all'],
    url='https://www.baidu.com',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Operating System :: OS Independent',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Programming Language :: Python',
        'Programming Language :: Python :: Implementation',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Topic :: Software Development :: Libraries'
    ],
)


# -*- coding:utf-8 -*- 
# Date: 2017-07-06 10:05:49
# Author: dekiven

# 参考：
# Python: Enum枚举的实现：http://www.cnblogs.com/codingmylife/archive/2013/05/31/3110656.html
# type函数创建新的类型（传入3个参数）： http://www.cnblogs.com/sesshoumaru/p/6129966.html
# SQLite - Python： http://www.runoob.com/sqlite/sqlite-python.html

# import sys 
# reload(sys)
# sys.setdefaultencoding('utf-8')

import os
import sqlite3

curDir = os.getcwd()

def EnumDef(*sequential, **named):
	'''
	创建枚举类型，可以通过该类型的reverse_mapping属性从值反向获取enum名
	详见参考：
		Python: 
			Enum枚举的实现：	http://www.cnblogs.com/codingmylife/archive/2013/05/31/3110656.html
			Python内置函数zip:	http://www.cnblogs.com/frydsh/archive/2012/07/10/2585370.html
			Python内置函数dict:	http://www.cnblogs.com/sesshoumaru/p/5990468.html	
			Python内置函数type:	https://docs.python.org/2/library/functions.html#type	

	# 测试代码
	EnumTest = EnumDef('Zero', One = 1, Two = 2)
	print(EnumTest.Zero)
	print(EnumTest.One)
	print(EnumTest.reverse_mapping[2])
	'''
	enums = dict(zip(sequential, range(len(sequential))), **named)
	reverse = dict((value, key) for key, value in enums.iteritems())
	enums['reverse_mapping'] = reverse
	return type('Enum', (), enums)



class TableKeyAtt(object):
	'''
	数据库字段属性
	一般数据采用的固定的静态数据类型，而SQLite采用的是动态数据类型，会根据存入值自动判断。SQLite具有以下五种数据类型：

	1.NULL：空值。
	2.INTEGER：带符号的整型，具体取决有存入数字的范围大小。
	3.REAL：浮点数字，存储为8-byte IEEE浮点数。
	4.TEXT：字符串文本。
	5.BLOB：二进制对象。

	但实际上，sqlite3也接受如下的数据类型：
	smallint 16 位元的整数。
	interger 32 位元的整数。
	decimal(p,s) p 精确值和 s 大小的十进位整数，精确值p是指全部有几个数(digits)大小值，s是指小数点後有几位数。如果没有特别指定，则系统会设为 p=5; s=0 。
	float  32位元的实数。
	double  64位元的实数。
	char(n)  n 长度的字串，n不能超过 254。
	varchar(n) 长度不固定且其最大长度为 n 的字串，n不能超过 4000。
	graphic(n) 和 char(n) 一样，不过其单位是两个字元 double-bytes， n不能超过127。这个形态是为了支援两个字元长度的字体，例如中文字。
	vargraphic(n) 可变长度且其最大长度为 n 的双字元字串，n不能超过 2000
	date  包含了 年份、月份、日期。
	time  包含了 小时、分钟、秒。
	timestamp 包含了 年、月、日、时、分、秒、千分之一秒。
	datetime 包含日期时间格式，必须写成'2010-08-05'不能写为'2010-8-5'，否则在读取时会产生错误！

	sqlite中char、varchar、text和nchar、nvarchar、ntext的区别
	1、CHAR。CHAR存储定长数据很方便，CHAR字段上的索引效率级高，比如定义char(10)，那么不论你存储的数据是否达到了10个字节，都要占去10个字节的空间,不足的自动用空格填充。
	2、VARCHAR。存储变长数据，但存储效率没有CHAR高。如果一个字段可能的值是不固定长度的，我们只知道它不可能超过10个字符，把它定义为 VARCHAR(10)是最合算的。VARCHAR类型的实际长度是它的值的实际长度+1。为什么“+1”呢？这一个字节用于保存实际使用了多大的长度。从空间上考虑，用varchar合适；从效率上考虑，用char合适，关键是根据实际情况找到权衡点。
	3、TEXT。text存储可变长度的非Unicode数据，最大长度为2^31-1(2,147,483,647)个字符。
	4、NCHAR、NVARCHAR、NTEXT。这三种从名字上看比前面三种多了个“N”。它表示存储的是Unicode数据类型的字符。我们知道字符中，英文字符只需要一个字节存储就足够了，但汉字众多，需要两个字节存储，英文与汉字同时存在时容易造成混乱，Unicode字符集就是为了解决字符集这种不兼容的问题而产生的，它所有的字符都用两个字节表示，即英文字符也是用两个字节表示。nchar、nvarchar的长度是在1到4000之间。和char、varchar比较起来，nchar、nvarchar则最多存储4000个字符，不论是英文还是汉字；而char、varchar最多能存储8000个英文，4000个汉字。可以看出使用nchar、nvarchar数据类型时不用担心输入的字符是英文还是汉字，较为方便，但在存储英文时数量上有些损失。
	所以一般来说，如果含有中文字符，用nchar/nvarchar，如果纯英文和数字，用char/varchar
	'''
	def __init__(self, keyName, typeStr = True, typeLen = 1, isDefautNull = True, isPrimaryKey = False):
		'''生成sql字段属性对象用于创建table
		参数：
		keyName  				字段名
		typeStr = True 			字段类型，默认为True，代表自增类型(同时也规定为主键和INT)；其他情况传入字符串表示的类型名
		typeLen = 1 			字段的长度，默认为1，该值在类型为char、vchar等的时候填写
		isDefautNull = True 	是否可为null，默认为可以
		isPrimaryKey = False	是否是主键，默认不是主键
		'''
		self.keyName = keyName
		if typeStr == True :
			self.typeStr = 'INTEGER'
			self.typeLen = 1
			self.isDefautNull = False
			self.isPrimaryKey = True
			self.autoincrement = True
		else :
			self.typeStr = typeStr
			self.typeLen = typeLen
			self.isDefautNull = isDefautNull
			self.isPrimaryKey = isPrimaryKey
			self.autoincrement = False

	def getSqlStr(self):
		keyName = self.keyName
		typeStr = self.typeStr
		typeLen = (self.typeLen > 1) and '(%d)'%(self.typeLen) or ''
		isDefautNull = self.isDefautNull and 'DEFAULT NULL' or 'NOT NULL'
		isPrimaryKey = self.isPrimaryKey and 'PRIMARY KEY' or ''
		auto = 'AUTOINCREMENT'
		if not self.autoincrement :
			return '%s %s%s %s %s' % (keyName, typeStr, typeLen, isPrimaryKey, isDefautNull)
		else :
			return '%s %s%s %s %s' % (keyName, typeStr, typeLen, isPrimaryKey, auto)

	def getIsPrimaryKey(self):
		return self.isPrimaryKey

class SQLiteHelper(object):
	'''
	SQLiteHelper
	'''
	def __init__(self, dbPath):
		super(SQLiteHelper, self).__init__()
		# TODO:在不存在的目录创建数据库会报错
		# 判断是否是绝对路径，创建文件夹
		# pDir = os.path.split(dbPath)
		self.conn = self.getConn(dbPath)
		self.cursor = self.getCursor()
		self.printDebug = False

	def getConn(self, path):
		if not os.path.isabs(path) :
			path = os.path.join(curDir, path)

		conn = sqlite3.connect(path)
		if (os.path.exists(path) and os.path.isfile(path)):
			return conn
		else :
			conn = None
			conn = sqlite3.connect(':memery:')
			return conn

	def getCursor(self, conn = None):		
		conn = conn or self.conn
		if conn is not None :
			return conn.cursor()
		else :
			# 如果传入的conn为空则返回在内存数据库中链接对象的游标
			self.conn = self.getConn('')
			return self.conn.cursor()

	def closeAll(self, conn = None, cursor = None):
		'''
		关闭数据库游标和数据库链接
		'''
		conn = conn or self.conn
		cursor = cursor or self.cursor

		try:
			if cursor is not None :
				cursor.close()
		finally:
			if conn is not None :
				conn.close()

	def createTable(self, tableName, keys, conn = None):
		'''
		创建数据表
		'''
		conn = conn or self.conn
		if isinstance(tableName, str) and tableName != '' :
			sql = 'CREATE TABLE IF NOT EXISTS %s \n(' % (tableName)
			count = 0
			if (isinstance(keys, list) or isinstance(keys, tuple)) :
				for key in keys :
					if isinstance(key, TableKeyAtt):
						sql = sql + key.getSqlStr() + ',\n'
						count += 1
				if count > 0 :
					sql = sql[:-2] + ');'
				else :
					sql = None
			elif isinstance(keys, str) and keys != '' :
				sql = keys
			else :
				sql = None

			if sql is not None :
				return self.execute(sql, conn = conn)

	def dropTable(self, tableName, commit = True, conn = None):
		'''
		删除数据表
		'''
		if isinstance(tableName, str) and tableName != '' :
			sql = 'DROP TABLE IF EXISTS %s' % (tableName)
			return self.execute(sql, conn = conn, commit = commit)

	def insert(self, tableName, data, commit = True, conn = None):
		'''
		添加数据
		'''
		if isinstance(data, list) or isinstance(data, tuple) and isinstance(tableName, str) and tableName != '':
			dataLen = len(data)
			sql = 'INSERT INTO %s VALUES (%s);' % (tableName, ('?,'*dataLen)[:-1])
			return self.execute(sql, data, conn, commit)

	def select(self, tableName, keyNames=(), where = None, conn = None ):
		'''
		查询数据,只传tableName表示查询整个表
		'''
		if isinstance(keyNames, list) or isinstance(keyNames, tuple) and isinstance(tableName, str) and tableName != '':
			sql = None
			whereStr, whereData = self.getWhereData(where)
			if len(keyNames) > 0 :
				sql = 'SELECT %s FROM %s %s;' % ((', '.join(keyNames)), tableName, whereStr)
			else :
				sql = 'SELECT %s FROM %s %s;' % ('*', tableName, whereStr)

			if sql is not None :
				print('select sql:%s'%(sql))
				return self.execute(sql, data = whereData, conn = conn, commit = False)

		return None

	def update(self, tableName, dataDic, where = None, commit = True, conn = None):
		'''
		更新数据
		'''
		if isinstance(dataDic, dict) and isinstance(tableName, str) and tableName != '':
			data = []
			setStr = ''
			for k, v in dataDic.items():
				setStr = setStr + '{}=(?), '.format(k)
				data.append(v)
			if len(setStr) > 2 :
				setStr = setStr[:-2]
			else :
				setStr = ''
				data = None

			whereStr, whereData = self.getWhereData(where)
			if data is not None and whereData is not None :
				whereData = data + whereData
			elif data is not None and whereData is None :
				whereData = data

			sql = 'UPDATE %s SET %s %s' % (tableName, setStr, whereStr)

			return self.execute(sql, data = whereData, conn = conn, commit = commit)

	def delete(self, tableName, where = None, commit = True, conn = None):
		'''
		删除数据
		'''
		if isinstance(tableName, str) and tableName != '':
			whereStr = ''
			# if isinstance(where, str) and where != '':
			# 	whereStr ='where ' + where

			whereStr, whereData = self.getWhereData(where)
			sql = 'DELETE FROM %s %s %s' % (tableName, whereStr)

			return self.execute(sql, data = whereData, conn = conn, commit = commit)

	def commit(self, conn = None):
		'''
		提交
		'''
		conn = conn or self.conn

		conn.commit()

	def rollback(self, conn = None):
		'''
		回滚自上一次调用 commit() 以来对数据库所做的更改。
		'''
		conn = conn or self.conn

		conn.rollback()

	def execute(self, sql, data = None, conn = None, commit = True):
		'''
		运行sql语句
		'''
		conn = conn or self.conn
		if isinstance(sql, str) and sql != '' :
			cursor = None
			if self.printDebug:
				print('execute, sql:\n\t"%s", \ndata:\n\t%s, \ncommit:\n\t%s'%(sql, str(data), str(commit)))
			try:
				if data is not None :
					cursor = conn.execute(sql, data)
				else :
					cursor = conn.execute(sql)

			except Exception as e:
				print(e)
			finally:
				if commit :
					conn.commit()
				return cursor 
		return None

	def getLastRow(self, tableName):
		'''获取某张表的最后一行
		'''
		return self.getAllData(tableName)[-1]

	def getAllData(self, tableName) :
		cursor = self.select(tableName)
		return self.fetchallCurser(cursor)

	def setPrintLog(self, value):
		self.printDebug = value

	def getTableHeaderNames(self, tableName) :
		cur = self.execute("select * from %s"%(tableName)) 
		if cur :
			col_name_list = [tuple[0] for tuple in cur.description]
			return col_name_list
		else :
			print('error: cursor is None, is there a table named "%s" ?'%(tableName))
			return ()

	# 静态方法必须添加该装饰器
	@staticmethod
	def getWhereData(where):
		'''where = None or where is str or where is dict like :{'name=':'testName', 'age>':18}
		'''
		if where is None :
			return '', None
		elif isinstance(where, dict) :
			wStr = 'WHERE '
			wData = []
			count = 0
			for k,v in where.items() :
				wStr += (count != 0 and 'AND ' or '') + k + '(?) '
				wData.append(v)
				count += 1
			return wStr, wData
		elif isinstance(where, str) :
			if where.lower().lstrip().find('where ') != 0 :
				where = 'WHERE ' + where
			return where, None


	@staticmethod
	def fetchallCurser(cursor):
		'''将cursor查询的所有行放到一个list
		'''
		if isinstance(cursor, sqlite3.Cursor) :
			return cursor.fetchall()
		return ()

	@staticmethod
	def fetchoneCurser(cursor):
		'''获取查询结果集中的下一行，返回一个单一的序列，当没有更多可用的数据时，则返回 None
		'''
		if isinstance(cursor, sqlite3.Cursor) :
			print('cursor.arraysize:%d'%(cursor.arraysize))
			return cursor.fetchone()
		return None

def test():
	print('test beging')

	test = SQLiteHelper('test.db')

	# keys = (
	# 	TableKeyAtt('id', 'INTEGER', isPrimaryKey = True),
	# 	TableKeyAtt('name', 'TEXT'),
	# 	TableKeyAtt('age', 'INTEGER'),
	# 	TableKeyAtt('score', 'REAL'),
	# )

	# test.createTable('test', keys)

	# # 添加数据
	# test.insert('test', (1, 'name1', 18, 60))
	# test.insert('test', (2, 'name2', 19, 60))
	# test.insert('test', (3, 'name3', 20, 60.2))
	# test.insert('test', (4, 'name4', 21, 60.3))

	# # 更新数据
	# test.update('test', {'age':99, 'name':'ha1hah'}, 'id = 1')

	# 删除一条数据
	# test.delete('test', 'id = 1')

	# 删除整个表所有数据
	# test.delete('test')

	# 查询数据
	# cursor = test.select('test', ('id', 'age' , 'name'))
	# data = cursor.fetchone()
	# while data is not None:
	# 	print(data)
	# 	data = cursor.fetchone()

	# 删除数据表
	# test.dropTable('test')

	# test.insert('test', (1, 'name1', 18, 60))

	# # 获取表的列名
	# l = test.getTableHeaderNames('test')
	# print(l)

	test.closeAll()

if __name__ == '__main__':
	test()
